'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _jsonPointer = require('json-pointer');

var _jsonPointer2 = _interopRequireDefault(_jsonPointer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

module.exports = function () {
  function Server(config) {
    var _this = this;

    _classCallCheck(this, Server);

    this.reset();

    this.name = config.name;
    this.port = config.port;
    this.dir = config.dir;

    this.express = (0, _express2.default)();
    this.express.get('*', function (req, res) {
      _this.getHandler(req, res);
    });
  }

  _createClass(Server, [{
    key: 'reset',
    value: function reset() {
      this.jsonMutations = [];
      this.statusMutations = [];
      this.respondToPath = null;
      this.responseHandler = function () {};
      return this;
    }
  }, {
    key: 'start',
    value: function start() {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        if (_this2.server) {
          return resolve();
        }
        _this2.server = _this2.express.listen(_this2.port, function () {
          resolve();
        });
      });
    }
  }, {
    key: 'stop',
    value: function stop() {
      if (this.server) {
        this.server.close();
        this.server = null;
      }
    }
  }, {
    key: 'getHandler',
    value: function getHandler(req, res) {
      var _this3 = this;

      var filePath = _path2.default.resolve(this.dir + req.path);

      // Allow all the things, security is overrated
      res.set('Access-Control-Allow-Origin', req.get('origin'));
      res.set('Access-Control-Allow-Credentials', true);

      // Get file and serve it
      this._fileExists(filePath).then(function () {
        return _this3._loadJson(filePath);
      }).then(function (json) {
        return _this3._validateJson(json, req, filePath);
      }).then(function (json) {
        return _this3._mutateJson(json, req);
      }).then(function (mutatedJson) {
        var status = _this3._getMutatedStatus(req);
        var body = mutatedJson;
        _this3.responseHandler(status, body);
        res.status(status).send(body);
      }).catch(function () {
        var err = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

        _this3.responseHandler(err[0], err[1]);
        res.status(err[0] || 500).send(err[1] || 'unknown error');
      });
    }
  }, {
    key: 'respondTo',
    value: function respondTo(path) {
      this.respondToPath = path;
      return this;
    }
  }, {
    key: 'andReplace',
    value: function andReplace(pointer, value) {
      this.jsonMutations.push({ path: this.respondToPath, pointer: pointer, value: value });
      return this;
    }
  }, {
    key: 'withStatus',
    value: function withStatus(status) {
      this.statusMutations.push({ path: this.respondToPath, status: status });
      return this;
    }
  }, {
    key: 'onResponse',
    value: function onResponse(fn) {
      this.responseHandler = fn;
      return this;
    }

    /*
     * Private helpers
     */

  }, {
    key: '_fileExists',
    value: function _fileExists(filePath) {
      return new Promise(function (resolve, reject) {
        _fs2.default.stat(filePath, function (err, stats) {
          err ? reject([404, err]) : resolve();
        });
      });
    }
  }, {
    key: '_loadJson',
    value: function _loadJson(filePath) {
      return new Promise(function (resolve, reject) {
        _fs2.default.readFile(filePath, 'utf8', function (err, data) {
          err ? reject([500, err]) : resolve(data);
        });
      });
    }
  }, {
    key: '_validateJson',
    value: function _validateJson(json, req, filePath) {
      if (!json.trim().startsWith('{')) {
        return Promise.resolve(json);
      }

      try {
        JSON.parse(json);
        return Promise.resolve(json);
      } catch (err) {
        console.log('Could not parse JSON:');
        console.trace(err);
        return Promise.reject([500, {
          message: 'File looks like JSON, but could not be parsed (' + err.message + ')',
          url: req.path,
          file: filePath
        }]);
      }
    }
  }, {
    key: '_mutateJson',
    value: function _mutateJson(json, req) {
      var mutated = JSON.parse(json);
      this.jsonMutations.forEach(function (mutation) {
        if (req.path == mutation.path) {
          _jsonPointer2.default.set(mutated, mutation.pointer, mutation.value);
        }
      });
      return Promise.resolve(mutated);
    }
  }, {
    key: '_getMutatedStatus',
    value: function _getMutatedStatus(req) {
      var mutationsForPath = this.statusMutations.filter(function (m) {
        return m.path == req.path;
      });
      if (mutationsForPath.length > 0) {
        return mutationsForPath[0].status;
      }
      return 200;
    }
  }]);

  return Server;
}();